import { httpApp } from "@/plugins/axiosApp";
import { store } from "@/store/store";
import { AudioPlayTime } from "@/types/student-client";

function resetExamQuestionDirty() {
  store.exam.examQuestionList = store.exam.examQuestionList.map((eq) => {
    return Object.assign({}, eq, { dirty: false });
  });
}

type Answer = {
  order: number;
  studentAnswer: string;
  audioPlayTimes: AudioPlayTime[];
  isStarred: boolean;
};

export async function answerAllQuestions(
  ignoreDirty?: boolean
): Promise<boolean> {
  const answers: Answer[] = store.exam.examQuestionList
    .filter((eq) => (ignoreDirty ? true : eq.dirty))
    .filter((eq) => eq.gotQuestionContent)
    .map((eq) => {
      return Object.assign(
        {
          order: eq.order,
          studentAnswer: eq.studentAnswer,
        },
        eq.audioPlayTimes && {
          audioPlayTimes: JSON.stringify(eq.audioPlayTimes) as any,
        },
        eq.isStarred && { isStarred: eq.isStarred }
      ) as Answer;
    });
  if (answers.length > 0) {
    try {
      await httpApp.post(
        "/api/ecs_oe_student/examQuestion/submitQuestionAnswer",
        answers
      );
      resetExamQuestionDirty();
    } catch (error) {
      logger({
        cnl: ["server", "local"],
        pgu: "AUTO",
        act: "提交答案失败",
        possibleError: error,
      });
      $message.error("提交答案失败");
      return false;
    }
  }
  // 提交成功，返回true，供最后提交时判断。自动提交失败，不暂停。
  return true;
}
